"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProjectDependencies = exports.determineLanguagesWithVersionUsed = exports.determineLanguagesUsed = exports.findPackageJson = exports.readPackageJson = void 0;
const tslib_1 = require("tslib");
const fs = tslib_1.__importStar(require("fs"));
const path_1 = require("path");
const resolve = tslib_1.__importStar(require("resolve"));
const core_programming_languages_1 = require("../core-programming-languages/core-programming-languages");
function getAllDependencies(pkg) {
    return new Set([
        ...Object.entries(pkg.dependencies || []),
        ...Object.entries(pkg.devDependencies || []),
        ...Object.entries(pkg.peerDependencies || []),
        ...Object.entries(pkg.optionalDependencies || []),
    ]);
}
function readPackageJson(packageJsonPath) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        try {
            return JSON.parse((yield fs.promises.readFile(packageJsonPath)).toString());
        }
        catch (_a) {
            return undefined;
        }
    });
}
exports.readPackageJson = readPackageJson;
function findPackageJson(workspaceDir, packageName) {
    try {
        // avoid require.resolve here, see: https://github.com/angular/angular-cli/pull/18610#issuecomment-681980185
        const packageJsonPath = resolve.sync(`${packageName}/package.json`, { basedir: workspaceDir });
        return packageJsonPath;
    }
    catch (_a) {
        return undefined;
    }
}
exports.findPackageJson = findPackageJson;
function determineLanguagesUsed(packageJsonMap) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const languagesUsedArr = [];
        const coreProgrammingLanguages = (0, core_programming_languages_1.coreProgrammingLanguagesMap)();
        // TODO
        for (const coreProgrammingLanguageKey in coreProgrammingLanguages) {
            if (packageJsonMap.has(coreProgrammingLanguageKey)) {
                const coreProgrammingLanguageValue = coreProgrammingLanguages[coreProgrammingLanguageKey];
                languagesUsedArr.push(coreProgrammingLanguageValue);
            }
        }
        return languagesUsedArr;
    });
}
exports.determineLanguagesUsed = determineLanguagesUsed;
function determineLanguagesWithVersionUsed(packageJsonMap) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const languagesUsedWithVersionArr = [];
        const coreProgrammingLanguages = (0, core_programming_languages_1.coreProgrammingLanguagesMap)();
        // TODO
        for (const coreProgrammingLanguageKey in coreProgrammingLanguages) {
            if (packageJsonMap.has(coreProgrammingLanguageKey)) {
                // we want to make the name non package json more human readable flavored
                // e.g. @angular/core is now angular
                const coreProgrammingLanguageValue = coreProgrammingLanguages[coreProgrammingLanguageKey];
                const packageJsonItem = packageJsonMap.get(coreProgrammingLanguageKey);
                // we always want to round patch to 0
                const [major, minor] = packageJsonItem.version.split('.').map(Number);
                const versionWithPathAtZero = `${major}.${minor}.0`;
                const coreProgrammingLanguageWithVersionValue = `${coreProgrammingLanguageValue}-${versionWithPathAtZero}`;
                languagesUsedWithVersionArr.push(coreProgrammingLanguageWithVersionValue);
            }
        }
        return languagesUsedWithVersionArr;
    });
}
exports.determineLanguagesWithVersionUsed = determineLanguagesWithVersionUsed;
function getProjectDependencies(dir) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const pkg = yield readPackageJson((0, path_1.join)(dir, 'package.json'));
        if (!pkg) {
            throw new Error('Could not find package.json');
        }
        const results = new Map();
        for (const [name, version] of getAllDependencies(pkg)) {
            const packageJsonPath = findPackageJson(dir, name);
            if (!packageJsonPath) {
                continue;
            }
            results.set(name, {
                name,
                version,
                path: (0, path_1.dirname)(packageJsonPath),
                package: yield readPackageJson(packageJsonPath),
            });
        }
        return results;
    });
}
exports.getProjectDependencies = getProjectDependencies;
